@@ -6,7 +6,7 @@ knitr::opts_chunk$set(
 
 ## -----------------------------------------------------------------------------
 N <- 1e7
-piR <- function(n, rng = runif) {
+piR <- function(n, rng = stats::runif) {
     x <- rng(n)
     y <- rng(n)
     4 * sum(sqrt(x^2 + y^2) < 1.0) / n
@@ -18,19 +18,19 @@ system.time(cat("pi ~= ", piR(N), "\n"))
 library(dqrng)
 dqRNGkind("Xoroshiro128+")
 dqset.seed(42)
-system.time(cat("pi ~= ", piR(N, rng = dqrunif), "\n"))
+system.time(cat("pi ~= ", piR(N, rng = dqrng::dqrunif), "\n"))
 
 ## -----------------------------------------------------------------------------
-system.time(runif(N))
-system.time(dqrunif(N))
+system.time(stats::runif(N))
+system.time(dqrng::dqrunif(N))
 
 ## -----------------------------------------------------------------------------
-system.time(rexp(N))
-system.time(dqrexp(N))
+system.time(stats::rexp(N))
+system.time(dqrng::dqrexp(N))
 
 ## -----------------------------------------------------------------------------
-system.time(rnorm(N))
-system.time(dqrnorm(N))
+system.time(stats::rnorm(N))
+system.time(dqrng::dqrnorm(N))
 
 ## -----------------------------------------------------------------------------
 system.time(for (i in 1:100)   sample.int(N, N/100, replace = TRUE))
