@@ -1,4 +1,5 @@
 // Copyright 2018-2019 Ralf Stubner (daqana GmbH)
+// Copyright 2022 Ralf Stubner
 //
 // This file is part of dqrng.
 //
@@ -19,12 +20,12 @@
 #include <Rcpp.h>
 #include <dqrng_generator.h>
 #include <dqrng_distribution.h>
+#include <dqrng_sample.h>
 #include <xoshiro.h>
 #include <pcg_random.hpp>
 #include <threefry.h>
 #include <convert_seed.h>
 #include <R_randgen.h>
-#include <minimal_int_set.h>
 
 namespace {
 dqrng::rng64_t init() {
@@ -145,66 +146,27 @@ double rexp(double rate = 1.0) {
   return rexp_impl();
 }
 
-// code for sampling
-namespace dqrng {
-namespace sample {
-template<int RTYPE, typename INT>
-inline Rcpp::Vector<RTYPE> replacement(INT m, INT n, int offset) {
-    using storage_t = typename Rcpp::traits::storage_type<RTYPE>::type;
-    Rcpp::Vector<RTYPE> result(Rcpp::no_init(n));
-    std::generate(result.begin(), result.end(),
-                  [m, offset] () {return static_cast<storage_t>(offset + (*rng)(m));});
-    return result;
-}
+//' @rdname dqrng-functions
+//' @export
+// [[Rcpp::export(rng = false)]]
+Rcpp::IntegerVector dqrrademacher(size_t n) {
+  Rcpp::IntegerVector res = Rcpp::no_init(n);
+  size_t k = 0;
+  for (size_t i = 0; i < ceil(n / 64.0) - 1; ++i) {
+    uint64_t bits = (*rng)();
 
-template<int RTYPE, typename INT>
-inline Rcpp::Vector<RTYPE> no_replacement_shuffle(INT m, INT n, int offset) {
-    using storage_t = typename Rcpp::traits::storage_type<RTYPE>::type;
-    Rcpp::Vector<RTYPE> tmp(Rcpp::no_init(m));
-    std::iota(tmp.begin(), tmp.end(), static_cast<storage_t>(offset));
-    for (INT i = 0; i < n; ++i) {
-        std::swap(tmp[i], tmp[i + (*rng)(m - i)]);
+    for (int j = 0; j <= 63; ++j, ++k) {
+      res[k] = ((bits >> j) & 1) * 2 - 1;
     }
-    if (m == n)
-        return tmp;
-    else
-        return Rcpp::Vector<RTYPE>(tmp.begin(), tmp.begin() + n);
-}
+  }
 
-template<int RTYPE, typename INT, typename SET>
-inline Rcpp::Vector<RTYPE> no_replacement_set(INT m, INT n, int offset) {
-    using storage_t = typename Rcpp::traits::storage_type<RTYPE>::type;
-    Rcpp::Vector<RTYPE> result(Rcpp::no_init(n));
-    SET elems(m, n);
-    for (INT i = 0; i < n; ++i) {
-        INT v = (*rng)(m);
-        while (!elems.insert(v)) {
-            v = (*rng)(m);
-        }
-        result(i) = static_cast<storage_t>(offset + v);
-    }
-    return result;
-}
+  uint64_t bits = (*rng)();
+  for (int j = 0; k < n; ++j, ++k) {
+    res[k] = ((bits >> j) & 1) * 2 - 1;
+  }
 
-template<int RTYPE, typename INT>
-inline Rcpp::Vector<RTYPE> sample(INT m, INT n, bool replace, int offset) {
-    if (replace || n <= 1) {
-        return dqrng::sample::replacement<RTYPE, INT>(m, n, offset);
-    } else {
-        if (!(m >= n))
-            Rcpp::stop("Argument requirements not fulfilled: m >= n");
-        if (m < 2 * n) {
-            return dqrng::sample::no_replacement_shuffle<RTYPE, INT>(m, n, offset);
-        } else if (m < 1000 * n) {
-            return dqrng::sample::no_replacement_set<RTYPE, INT, dqrng::minimal_bit_set>(m, n, offset);
-        } else {
-            return dqrng::sample::no_replacement_set<RTYPE, INT, dqrng::minimal_hash_set<INT>>(m, n, offset);
-        }
-    }
+  return res;
 }
-} // sample
-} // dqrng
-
 
 // [[Rcpp::export(rng = false)]]
 Rcpp::IntegerVector dqsample_int(int m,
@@ -214,7 +176,7 @@ Rcpp::IntegerVector dqsample_int(int m,
                                  int offset = 0) {
     if (!(m > 0 && n >= 0))
         Rcpp::stop("Argument requirements not fulfilled: m > 0 && n >= 0");
-    return dqrng::sample::sample<INTSXP, uint32_t>(uint32_t(m), uint32_t(n), replace, offset);
+    return dqrng::sample::sample<INTSXP, uint32_t>(rng, uint32_t(m), uint32_t(n), replace, offset);
 }
 
 // [[Rcpp::export(rng = false)]]
@@ -228,6 +190,6 @@ Rcpp::NumericVector dqsample_num(double m,
 #else
     if (!(m > 0 && n >= 0))
         Rcpp::stop("Argument requirements not fulfilled: m > 0 && n >= 0");
-    return dqrng::sample::sample<REALSXP, uint64_t>(uint64_t(m), uint64_t(n), replace, offset);
+    return dqrng::sample::sample<REALSXP, uint64_t>(rng, uint64_t(m), uint64_t(n), replace, offset);
 #endif
 }
